/*===========================================================================*\
*                                                                            *
*                              OpenFlipper                                   *
 *           Copyright (c) 2001-2015, RWTH-Aachen University                 *
 *           Department of Computer Graphics and Multimedia                  *
 *                          All rights reserved.                             *
 *                            www.openflipper.org                            *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * This file is part of OpenFlipper.                                         *
 *---------------------------------------------------------------------------*
 *                                                                           *
 * Redistribution and use in source and binary forms, with or without        *
 * modification, are permitted provided that the following conditions        *
 * are met:                                                                  *
 *                                                                           *
 * 1. Redistributions of source code must retain the above copyright notice, *
 *    this list of conditions and the following disclaimer.                  *
 *                                                                           *
 * 2. Redistributions in binary form must reproduce the above copyright      *
 *    notice, this list of conditions and the following disclaimer in the    *
 *    documentation and/or other materials provided with the distribution.   *
 *                                                                           *
 * 3. Neither the name of the copyright holder nor the names of its          *
 *    contributors may be used to endorse or promote products derived from   *
 *    this software without specific prior written permission.               *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED *
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A           *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER *
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,  *
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        *
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    *
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        *
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              *
*                                                                            *
\*===========================================================================*/


//=============================================================================
//
//  CLASS InfoVolumeMeshObjectPlugin - IMPLEMENTATION
//
//=============================================================================


//== INCLUDES =================================================================

#include "VolumeMeshObjectInfoPlugin.hh"
#include "OpenFlipper/common/GlobalOptions.hh"
#include "OpenFlipper/BasePlugin/PluginFunctions.hh"
#include "Type-OpenVolumeMesh/ObjectTypes/HexahedralMesh/PluginFunctionsHexahedralMesh.hh"
#include "Type-OpenVolumeMesh/ObjectTypes/TetrahedralMesh/PluginFunctionsTetrahedralMesh.hh"
#include "Type-OpenVolumeMesh/ObjectTypes/PolyhedralMesh/PluginFunctionsPolyhedralMesh.hh"

//== IMPLEMENTATION ==========================================================


InfoVolumeMeshObjectPlugin::InfoVolumeMeshObjectPlugin()
{
}

InfoVolumeMeshObjectPlugin::~InfoVolumeMeshObjectPlugin()
{
}

void InfoVolumeMeshObjectPlugin::updateInfoBar()
{
  if (!infoBar_) {
      return;
  }
  if ( PluginFunctions::targetCount() != 1) {
      infoBar_->hideCounts();
      return;
  }
  BaseObjectData *target = nullptr;
  DataType supported(DATA_TETRAHEDRAL_MESH | DATA_POLYHEDRAL_MESH | DATA_HEXAHEDRAL_MESH);
  for (auto *obj: PluginFunctions::objects(PluginFunctions::TARGET_OBJECTS, supported))
  {
      target = obj;
      break;
  }

  if (!target) {
      infoBar_->hideCounts();
      return;
  }
  auto *pmo = PluginFunctions::polyhedralMeshObject(target);
  auto *hmo = PluginFunctions::hexahedralMeshObject(target);
  auto *tmo = PluginFunctions::tetrahedralMeshObject(target);
  if (pmo && pmo->mesh()) {
      infoBar_->showCounts("Poly", pmo->mesh());
  } else if (hmo && hmo->mesh()) {
      infoBar_->showCounts("Hex", hmo->mesh());
  } else if (tmo && tmo->mesh()) {
      infoBar_->showCounts("Tet", tmo->mesh());
  } else {
      infoBar_->hideCounts();
  }
}


void InfoVolumeMeshObjectPlugin::initializePlugin()
{
}

/// initialize the plugin
void InfoVolumeMeshObjectPlugin::pluginsInitialized()
{
  setDescriptions();

  if ( OpenFlipper::Options::gui()) {

    infoBar_ = new VolumeMeshInfoBar();

    emit addWidgetToStatusbar(infoBar_);
  }
}

//------------------------------------------------------------------------------

void InfoVolumeMeshObjectPlugin::slotObjectUpdated( int _identifier , const UpdateType& _type)
{
  updateInfoBar();
}

//------------------------------------------------------------------------------

void InfoVolumeMeshObjectPlugin::slotObjectSelectionChanged( int _identifier )
{
  slotObjectUpdated( _identifier , UPDATE_ALL );
}

//------------------------------------------------------------------------------

void InfoVolumeMeshObjectPlugin::slotAllCleared()
{
}


