/*===========================================================================*\
*                                                                            *
*                              OpenFlipper                                   *
 *           Copyright (c) 2001-2015, RWTH-Aachen University                 *
 *           Department of Computer Graphics and Multimedia                  *
 *                          All rights reserved.                             *
 *                            www.openflipper.org                            *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * This file is part of OpenFlipper.                                         *
 *---------------------------------------------------------------------------*
 *                                                                           *
 * Redistribution and use in source and binary forms, with or without        *
 * modification, are permitted provided that the following conditions        *
 * are met:                                                                  *
 *                                                                           *
 * 1. Redistributions of source code must retain the above copyright notice, *
 *    this list of conditions and the following disclaimer.                  *
 *                                                                           *
 * 2. Redistributions in binary form must reproduce the above copyright      *
 *    notice, this list of conditions and the following disclaimer in the    *
 *    documentation and/or other materials provided with the distribution.   *
 *                                                                           *
 * 3. Neither the name of the copyright holder nor the names of its          *
 *    contributors may be used to endorse or promote products derived from   *
 *    this software without specific prior written permission.               *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED *
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A           *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER *
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,  *
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,       *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR        *
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    *
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        *
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              *
*                                                                            *
\*===========================================================================*/




#pragma once

#include "ui_textureProperties.h"
#include "TextureData.hh"

#ifdef WITH_QWT
  #include "QwtFunctionPlot.hh"
#endif

/** \class texturePropertiesWidget
 *
 * Widget for setting different properties of textures
 */
class texturePropertiesWidget : public QDialog, public Ui::Dialog
{
  Q_OBJECT

  signals:
    void applyProperties(TextureData* _texData, QString _textureName, int _id);

    void getCoordinates1D(QString _textureName, int _id, std::vector< double >& _x );

  public:
    explicit texturePropertiesWidget(QWidget *parent = 0);

    void show(TextureData* _texData, int _id, QString _name = "");

  private slots:

    void textureAboutToChange(QTreeWidgetItem* _item , int _column);

    void textureChanged(QTreeWidgetItem* _item , int _column);

    void slotButtonBoxClicked(QAbstractButton* _button);

    void slotPropertiesChanged(double _value = 0.0);

    void slotChangeImage();

  private:

    bool             propChanged_;
    QTreeWidgetItem* curItem_;
    QString          currentImage_;
    QImage           image_;

    TextureData*     texData_;
    QString          textureName_;
    int              id_;

#ifdef WITH_QWT
    ACG::QwtFunctionPlot* functionPlot_;
#endif

};

